import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'package:iroute/iroute.dart';
import 'iroute_state.dart';
import 'iroute_common.dart';

///isRoot:顶层的路由
///pageList:如果为null，则顶层路由是单层的
class IRoute extends RouterDelegate<String>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin {
  final GlobalKey<NavigatorState> navigatorKey;

  PageList? pageList;
  int get selectedPage {
    return pageList!.current;
  }

  bool isRoot = true;

  set selectedPage(int value) {
    if (pageList!.current == value) return;
    pageList!.current = value;
    //print(
    //    'nested: selectPage,url in pageList is ${pageList.pages[pageList.current].currentUrl}');
    push(pageList!.pages[pageList!.current].currentUrl);
  }

  List<Page> _pages = [];
  IRouteState iRouteState = IRouteState();
  Map<String, RouteBuilder> get routes => IRouteState().routes;
  bool get canPop => iRouteState.canPop;
  bool initing = false;

  IRoute(
      {this.pageList,
      required Map<String, RouteBuilder> routes,
      this.isRoot = true})
      : navigatorKey = GlobalKey<NavigatorState>() {
    //在这里初始化routeState，否则必须在main中执行。
    if (!IRouteState.inited) IRouteState.init(routes);
    if (isRoot) iRouteState.addListener(notifyListeners);
  }

  static IRoute of(BuildContext context) {
    final delegate = Router.of(context).routerDelegate;
    assert(delegate is IRoute, 'Delegate type must match');
    return delegate as IRoute;
  }

  @override
  Future<void> setInitialRoutePath(String configuration) {
    //print('SetInit,path=$configuration');
    return super.setInitialRoutePath(configuration);
  }

  //如果指定了child
  //如果没有，则pages
  @override
  Widget build(BuildContext context) {
    if (!isRoot) {
      pageList!.changeToUrl(iRouteState.currentConfiguration);
    }

    //只加入最后一条，则scaffold的后退按钮将不再显示，与canPop无关
    //_pages = [iRouteState.stack.last];

    //_pages不变化，则不显示
    //由此，直接赋予iRouteState.stack，则页面不会更新。
    //nested可以更新，是由于整个appShell重绘。重绘的时候，可能出现错误url进入某页面。
    _pages = iRouteState.stack.toList();
    //_pages = iRouteState.stack;

    //print('iRoute:build , url=$currentConfiguration');
    return Navigator(
        key: navigatorKey,
        pages: (isRoot && pageList != null)
            ? [
                MaterialPage(
                  key: ValueKey('AppShell'), //UniqueKey(),
                  child: IRouteAppShell(
                    pageList: this.pageList!,
                    routes: this.routes,
                    iRouteState: iRouteState,
                  ),
                ),
              ]
            : _pages,
        onPopPage: (route, result) {
          if (!route.didPop(result)) {
            return false;
          }
          if (canPop) {
            pop();
          }
          return true;
        });
  }

  @override
  String? get currentConfiguration {
    if (!isRoot) {
      return null;
    } else {
      //print('current url=${iRouteState.currentConfiguration}');
      return iRouteState.currentConfiguration;
    }
  }

  @override
  dispose() {
    iRouteState.removeListener(notifyListeners);
    super.dispose();
  }

  push(String path) {
    //print('IRoute: push 准备push:$path');
    if (path == currentConfiguration) return;
    if (isRoot) {
      iRouteState.push(path);
      //notifyListeners();
    } else {
      String url = path;
      if (!path.contains(':ipage') && path != '/') {
        url = url + ':ipage=${pageList!.current}';
      }
      pageList!.pages[pageList!.current].currentUrl = url;
      iRouteState.push(url);
    }
  }

  pop() {
    if (!canPop) return;
    //print('iroute pop');
    iRouteState.pop();
    //notifyListeners();
  }

  // @override
  // Future<void> setInitialRoutePath(String configuration) {
  //   initing = true;
  //   return super.setInitialRoutePath(configuration);
  // }

  //setNewPath不需要notifyListeners()
  //注意：两种情况会执行这里
  //首先是setInitialRoutePath之后，我们可设置initing字段为true，而这里将其改为false
  //其次是响应浏览器地址栏变化，包括前进、后退、选历史记录、修改浏览器地址，此时initing=false
  //此时可注意，不向浏览器发送历史记录，如Router.neglect
  @override
  Future<void> setNewRoutePath(String path) async {
    //print('setNewPath:path=$path,stack=${iRouteState.stack.toString()}');
    if (isRoot) {
      iRouteState.setNewRoutePath(path); //加入path，且通知
      //return SynchronousFuture(null);
    } else {
      assert(false);
    }
  }
}
